/**
 * Select text between two wrapper characters.
 *
 * @category  WeBuilder Plugin
 * @package   Select Between
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2016
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.02
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     34   function SelectBetwenDoubleQuotes()
 *     43   function SelectBetwenSingleQuotes()
 *     52   function SelectBetwenAngleBrackets()
 *     61   function SelectBetwenCurlyBrackets()
 *     70   function SelectBetwenRoundBrackets()
 *     79   function SelectBetwenSquareBrackets()
 *     91   function SelectBetween(startChar, endChar)
 *    132   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 8
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

/**
 * Select between double quotes
 *
 * @return void
 */
function SelectBetwenDoubleQuotes() {
    SelectBetween("\"","\"");
}

/**
 * Select between single quotes
 *
 * @return void
 */
function SelectBetwenSingleQuotes() {
    SelectBetween("'","'");
}

/**
 * Select between angle brackets
 *
 * @return void
 */
function SelectBetwenAngleBrackets() {
    SelectBetween("<",">");
}

/**
 * Select between curly brackets
 *
 * @return void
 */
function SelectBetwenCurlyBrackets() {
    SelectBetween("\\\{","\\\}");
}

/**
 * Select between round brackets
 *
 * @return void
 */
function SelectBetwenRoundBrackets() {
    SelectBetween("\\\(","\\\)");
}

/**
 * Select between square brackets
 *
 * @return void
 */
function SelectBetwenSquareBrackets() {
    SelectBetween("\\\[","\\\]");
}

/**
 * Select text between two wrapper charcters
 *
 * @param  string   startChar
 * @param  string   endChar
 *
 * @return void
 */
function SelectBetween(startChar, endChar) {
    var text = Editor.Text;
    if (Script.ReadSetting("Include escaped wrapper chars", "1") == "1") {
        // Convert escaped startChar/endChars into "--" in order to make those part of selection
        text = RegexReplace(RegexReplace(text ,"\\\\" + startChar, "--", true),"\\\\" + endChar, "--", true);
    }

    // Convert line/column format into text position
    Editor.CursorPosToTextPos(Editor.Selection.SelStartColReal, Editor.Selection.SelStartLine, textPos);

    // Test if we can find the startChar at the left side of the cursor position
    var leftContent = Copy(text, 0, textPos);
    if (RegexMatchAll(leftContent, "[\\s\\S]*" + startChar + "(?=[^" + startChar + "]*)", true, match, p) == true) {

        var startPos = Length(_v(match, [0, 0]));

        // Test if we can find the endChar at the right side of the cursor position
        if (RegexMatchAll(Copy(text, textPos + 1, Length(text)), "[^" + endChar + "]*(" + endChar + ")", true, match, p) == true) {

            var endPos = Length(leftContent) + _v(p, [0, 1]) - 1;

            // Convert text positions into lines/columns format
            Editor.TextPosToCursorPos(startPos, startX, startY);
            Editor.TextPosToCursorPos(endPos, endX, endY);

            // Make selection
            var Sel = Editor.Selection;
            Sel.SelStartLine = startY;
            Sel.SelStartCol = startX;
            Sel.SelEndLine = endY;
            Sel.SelEndCol = endX;
            Editor.Selection = Sel;
        }
    }
}

/**
 * Show info when plugin is installed.
 *
 * @return void
 */
function OnInstalled() {
  Alert("Select Between 1.02 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");

var bmp = new TBitmap;

if (Script.ReadSetting("Enable Select Between Double Quotes", "1") == "1") {
    var act1 = Script.RegisterDocumentAction("Select Between", "Select Between Double Quotes", "Select Between", "SelectBetwenDoubleQuotes");
    LoadFileToBitmap(Script.Path + "double_quote.png", bmp);
    Actions.SetIcon(act1, bmp);
}

if (Script.ReadSetting("Enable Select Between Single Quotes", "1") == "1") {
    var act2 = Script.RegisterDocumentAction("Select Between", "Select Between Single Quotes", "Select Between", "SelectBetwenSingleQuotes");
    LoadFileToBitmap(Script.Path + "single_quote.png", bmp);
    Actions.SetIcon(act2, bmp);
}

if (Script.ReadSetting("Enable Select Between Angle Brackets", "1") == "1") {
    var act3 = Script.RegisterDocumentAction("Select Between", "Select Between Angle Brackets", "Select Between", "SelectBetwenAngleBrackets");
    LoadFileToBitmap(Script.Path + "angle_bracket.png", bmp);
    Actions.SetIcon(act3, bmp);
}

if (Script.ReadSetting("Enable Select Between Curly Brackets", "1") == "1") {
    var act4 = Script.RegisterDocumentAction("Select Between", "Select Between Curly Brackets", "Select Between", "SelectBetwenCurlyBrackets");
    LoadFileToBitmap(Script.Path + "curly_bracket.png", bmp);
    Actions.SetIcon(act4, bmp);
}

if (Script.ReadSetting("Enable Select Between Round Brackets", "1") == "1") {
    var act5 = Script.RegisterDocumentAction("Select Between", "Select Between Round Brackets", "Select Between", "SelectBetwenRoundBrackets");
    LoadFileToBitmap(Script.Path + "round_bracket.png", bmp);
    Actions.SetIcon(act5, bmp);
}

if (Script.ReadSetting("Enable Select Between Square Brackets", "1") == "1") {
    var act6 = Script.RegisterDocumentAction("Select Between", "Select Between Square Brackets", "Select Between", "SelectBetwenSquareBrackets");
    LoadFileToBitmap(Script.Path + "square_bracket.png", bmp);
    Actions.SetIcon(act6, bmp);
}

delete bmp;